@@ -1,6 +1,6 @@
 /* findcmd.h - functions from findcmd.c. */
 
-/* Copyright (C) 1997-2012 Free Software Foundation, Inc.
+/* Copyright (C) 1997-2015 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -23,13 +23,25 @@
 
 #include "stdc.h"
 
+/* Flags for search_for_command */
+#define CMDSRCH_HASH		0x01
+#define CMDSRCH_STDPATH		0x02
+#define CMDSRCH_TEMPENV		0x04
+
 extern int file_status __P((const char *));
 extern int executable_file __P((const char *));
 extern int is_directory __P((const char *));
 extern int executable_or_directory __P((const char *));
 extern char *find_user_command __P((const char *));
+extern char *find_in_path __P((const char *, char *, int));
 extern char *find_path_file __P((const char *));
 extern char *search_for_command __P((const char *, int));
 extern char *user_command_matches __P((const char *, int, int));
+extern void setup_exec_ignore __P((char *));
+
+extern int dot_found_in_search;
+
+/* variables managed via shopt */
+extern int check_hashed_filenames;
 
 #endif /* _FINDCMD_H_ */
