@@ -23,6 +23,8 @@
 
 #include "stdc.h"
 
+extern int here_doc_first_line;
+
 extern void cmd_init __P((void));
 
 extern WORD_DESC *alloc_word_desc __P((void));
@@ -47,7 +49,7 @@ extern COMMAND *make_until_command __P((COMMAND *, COMMAND *));
 extern COMMAND *make_bare_simple_command __P((void));
 extern COMMAND *make_simple_command __P((ELEMENT, COMMAND *));
 extern void make_here_document __P((REDIRECT *, int));
-extern REDIRECT *make_redirection __P((int, enum r_instruction, REDIRECTEE));
+extern REDIRECT *make_redirection __P((REDIRECTEE, enum r_instruction, REDIRECTEE, int));
 extern COMMAND *make_function_def __P((WORD_DESC *, COMMAND *, int, int));
 extern COMMAND *clean_simple_command __P((COMMAND *));
 
