@@ -1,6 +1,6 @@
 /* variables.h -- data structures for shell variables. */
 
-/* Copyright (C) 1987-2010 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2018 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -112,8 +112,9 @@ typedef struct _vlist {
 #define att_uppercase	0x0000100	/* word converted to uppercase on assignment */
 #define att_lowercase	0x0000200	/* word converted to lowercase on assignment */
 #define att_capcase	0x0000400	/* word capitalized on assignment */
+#define att_nameref	0x0000800	/* word is a name reference */
 
-#define user_attrs	(att_exported|att_readonly|att_integer|att_local|att_trace|att_uppercase|att_lowercase|att_capcase)
+#define user_attrs	(att_exported|att_readonly|att_integer|att_local|att_trace|att_uppercase|att_lowercase|att_capcase|att_nameref)
 
 #define attmask_user	0x0000fff
 
@@ -124,6 +125,7 @@ typedef struct _vlist {
 #define att_imported	0x0008000	/* came from environment */
 #define att_special	0x0010000	/* requires special handling */
 #define att_nofree	0x0020000	/* do not free value on unset */
+#define att_regenerate	0x0040000	/* regenerate when exported */
 
 #define	attmask_int	0x00ff000
 
@@ -144,6 +146,7 @@ typedef struct _vlist {
 #define uppercase_p(var)	((((var)->attributes) & (att_uppercase)))
 #define lowercase_p(var)	((((var)->attributes) & (att_lowercase)))
 #define capcase_p(var)		((((var)->attributes) & (att_capcase)))
+#define nameref_p(var)		((((var)->attributes) & (att_nameref)))
 
 #define invisible_p(var)	((((var)->attributes) & (att_invisible)))
 #define non_unsettable_p(var)	((((var)->attributes) & (att_nounset)))
@@ -151,23 +154,33 @@ typedef struct _vlist {
 #define imported_p(var)		((((var)->attributes) & (att_imported)))
 #define specialvar_p(var)	((((var)->attributes) & (att_special)))
 #define nofree_p(var)		((((var)->attributes) & (att_nofree)))
+#define regen_p(var)		((((var)->attributes) & (att_regenerate)))
 
 #define tempvar_p(var)		((((var)->attributes) & (att_tempvar)))
+#define propagate_p(var)	((((var)->attributes) & (att_propagate)))
+
+/* Variable names: lvalues */
+#define name_cell(var)		((var)->name)
 
 /* Acessing variable values: rvalues */
 #define value_cell(var)		((var)->value)
 #define function_cell(var)	(COMMAND *)((var)->value)
 #define array_cell(var)		(ARRAY *)((var)->value)
 #define assoc_cell(var)		(HASH_TABLE *)((var)->value)
+#define nameref_cell(var)	((var)->value)		/* so it can change later */
+
+#define NAMEREF_MAX	8	/* only 8 levels of nameref indirection */
 
-#define var_isnull(var)		((var)->value == 0)
 #define var_isset(var)		((var)->value != 0)
+#define var_isunset(var)	((var)->value == 0)
+#define var_isnull(var)		((var)->value && *(var)->value == 0)
 
 /* Assigning variable values: lvalues */
 #define var_setvalue(var, str)	((var)->value = (str))
 #define var_setfunc(var, func)	((var)->value = (char *)(func))
 #define var_setarray(var, arr)	((var)->value = (char *)(arr))
 #define var_setassoc(var, arr)	((var)->value = (char *)(arr))
+#define var_setref(var, str)	((var)->value = (str))
 
 /* Make VAR be auto-exported. */
 #define set_auto_export(var) \
@@ -205,6 +218,15 @@ typedef struct _vlist {
 	      (var)->exportstr = (char *)NULL; \
 	    } \
 	} while (0)
+
+#define ifsname(s)	((s)[0] == 'I' && (s)[1] == 'F' && (s)[2] == 'S' && (s)[3] == '\0')
+
+/* Flag values for make_local_variable */
+#define MKLOC_INHERIT		0x01
+
+/* Special value for nameref with invalid value for creation or assignment */
+extern SHELL_VAR nameref_invalid_value;
+#define INVALID_NAMEREF_VALUE	(void *)&nameref_invalid_value
 	
 /* Stuff for hacking variables. */
 typedef int sh_var_map_func_t __P((SHELL_VAR *));
@@ -220,7 +242,18 @@ extern int variable_context;
 extern char *dollar_vars[];
 extern char **export_env;
 
+extern int tempenv_assign_error;
+extern int array_needs_making;
+extern int shell_level;
+
+/* XXX */
+extern WORD_LIST *rest_of_args;
+extern pid_t dollar_dollar_pid;
+
 extern void initialize_shell_variables __P((char **, int));
+
+extern int validate_inherited_value __P((SHELL_VAR *, int));
+
 extern SHELL_VAR *set_if_not __P((char *, char *));
 
 extern void sh_set_lines_and_columns __P((int, int));
@@ -233,15 +266,29 @@ extern SHELL_VAR *var_lookup __P((const char *, VAR_CONTEXT *));
 extern SHELL_VAR *find_function __P((const char *));
 extern FUNCTION_DEF *find_function_def __P((const char *));
 extern SHELL_VAR *find_variable __P((const char *));
-extern SHELL_VAR *find_variable_internal __P((const char *, int));
-extern SHELL_VAR *find_tempenv_variable __P((const char *));
+extern SHELL_VAR *find_variable_noref __P((const char *));
+extern SHELL_VAR *find_variable_last_nameref __P((const char *, int));
+extern SHELL_VAR *find_global_variable_last_nameref __P((const char *, int));
+extern SHELL_VAR *find_variable_nameref __P((SHELL_VAR *));
+extern SHELL_VAR *find_variable_nameref_for_create __P((const char *, int));
+extern SHELL_VAR *find_variable_nameref_for_assignment __P((const char *, int));
+/*extern SHELL_VAR *find_variable_internal __P((const char *, int));*/
+extern SHELL_VAR *find_variable_tempenv __P((const char *));
+extern SHELL_VAR *find_variable_notempenv __P((const char *));
 extern SHELL_VAR *find_global_variable __P((const char *));
+extern SHELL_VAR *find_global_variable_noref __P((const char *));
+extern SHELL_VAR *find_shell_variable __P((const char *));
+extern SHELL_VAR *find_tempenv_variable __P((const char *));
+extern SHELL_VAR *find_variable_no_invisible __P((const char *));
+extern SHELL_VAR *find_variable_for_assignment __P((const char *));
+extern char *nameref_transform_name __P((char *, int));
 extern SHELL_VAR *copy_variable __P((SHELL_VAR *));
-extern SHELL_VAR *make_local_variable __P((const char *));
+extern SHELL_VAR *make_local_variable __P((const char *, int));
 extern SHELL_VAR *bind_variable __P((const char *, char *, int));
+extern SHELL_VAR *bind_global_variable __P((const char *, char *, int));
 extern SHELL_VAR *bind_function __P((const char *, COMMAND *));
 
-extern void bind_function_def __P((const char *, FUNCTION_DEF *));
+extern void bind_function_def __P((const char *, FUNCTION_DEF *, int));
 
 extern SHELL_VAR **map_over __P((sh_var_map_func_t *, VAR_CONTEXT *));
 SHELL_VAR **map_over_funcs __P((sh_var_map_func_t *));
@@ -267,14 +314,18 @@ extern char *sh_get_env_value __P((const char *));
 extern char *make_variable_value __P((SHELL_VAR *, char *, int));
 
 extern SHELL_VAR *bind_variable_value __P((SHELL_VAR *, char *, int));
-extern SHELL_VAR *bind_int_variable __P((char *, char *));
+extern SHELL_VAR *bind_int_variable __P((char *, char *, int));
 extern SHELL_VAR *bind_var_to_int __P((char *, intmax_t));
 
 extern int assign_in_env __P((WORD_DESC *, int));
 
 extern int unbind_variable __P((const char *));
+extern int check_unbind_variable __P((const char *));
+extern int unbind_nameref __P((const char *));
+extern int unbind_variable_noref __P((const char *));
 extern int unbind_func __P((const char *));
 extern int unbind_function_def __P((const char *));
+extern int delete_var __P((const char *, VAR_CONTEXT *));
 extern int makunbound __P((const char *, VAR_CONTEXT *));
 extern int kill_local_variable __P((const char *));
 extern void delete_all_variables __P((HASH_TABLE *));
@@ -293,6 +344,8 @@ extern void push_dollar_vars __P((void));
 extern void pop_dollar_vars __P((void));
 extern void dispose_saved_dollar_vars __P((void));
 
+extern void init_bash_argv __P((void));
+extern void save_bash_argv __P((void));
 extern void push_args __P((WORD_LIST *));
 extern void pop_args __P((void));
 
@@ -303,6 +356,7 @@ extern void dispose_used_env_vars __P((void));
 extern void dispose_function_env __P((void));
 extern void dispose_builtin_env __P((void));
 extern void merge_temporary_env __P((void));
+extern void flush_temporary_env __P((void));
 extern void merge_builtin_env __P((void));
 extern void kill_all_local_variables __P((void));
 
@@ -327,10 +381,10 @@ extern void print_var_function __P((SHELL_VAR *));
 
 #if defined (ARRAY_VARS)
 extern SHELL_VAR *make_new_array_variable __P((char *));
-extern SHELL_VAR *make_local_array_variable __P((char *));
+extern SHELL_VAR *make_local_array_variable __P((char *, int));
 
 extern SHELL_VAR *make_new_assoc_variable __P((char *));
-extern SHELL_VAR *make_local_assoc_variable __P((char *));
+extern SHELL_VAR *make_local_assoc_variable __P((char *, int));
 
 extern void set_pipestatus_array __P((int *, int));
 extern ARRAY *save_pipestatus_array __P((void));
@@ -355,6 +409,7 @@ extern void sv_ifs __P((char *));
 extern void sv_path __P((char *));
 extern void sv_mail __P((char *));
 extern void sv_funcnest __P((char *));
+extern void sv_execignore __P((char *));
 extern void sv_globignore __P((char *));
 extern void sv_ignoreeof __P((char *));
 extern void sv_strict_posix __P((char *));
@@ -362,6 +417,7 @@ extern void sv_optind __P((char *));
 extern void sv_opterr __P((char *));
 extern void sv_locale __P((char *));
 extern void sv_xtracefd __P((char *));
+extern void sv_shcompat __P((char *));
 
 #if defined (READLINE)
 extern void sv_comp_wordbreaks __P((char *));
@@ -384,8 +440,12 @@ extern void sv_histchars __P((char *));
 extern void sv_histtimefmt __P((char *));
 #endif /* HISTORY */
 
-#if defined (HAVE_TZSET) && defined (PROMPT_STRING_DECODE)
+#if defined (HAVE_TZSET)
 extern void sv_tz __P((char *));
 #endif
 
+#if defined (JOB_CONTROL)
+extern void sv_childmax __P((char *));
+#endif
+
 #endif /* !_VARIABLES_H_ */
